/*
 * File Archive Library
 * Copyright (C) 2000-2001 Patrizio Bruno <patrizio@dada.it>
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation; either
 *    version 2.1 of the License, or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    Read the file LICENSE for more.
 *
 */

/*
 * $Log: unarchive.c,v $
 * Revision 1.13  2001/05/28 11:44:33  rusty
 *
 * fixed a bug handling entry_info.st_* on solaris
 *
 * Revision 1.12  2001/05/25 15:28:35  rusty
 *
 * fixed the struct dirent handling on solaris
 *
 * Revision 1.11  2001/05/21 12:16:45  rusty
 *
 * added a md5 checksum for the whole archive
 *
 * Revision 1.10  2001/05/21 11:06:31  rusty
 *
 * byte ordering fixed
 *
 * Revision 1.9  2001/05/16 16:08:17  rusty
 * *** empty log message ***
 *
 * Revision 1.8  2001/05/16 11:41:13  rusty
 *
 * code clean up
 *
 * Revision 1.7  2001/05/16 11:04:57  rusty
 *
 * some debug code removed
 * added a function displaying the internal version of the library: far_version() in utils.c
 *
 * Revision 1.6  2001/05/16 10:50:39  rusty
 *
 * fixed the checksum control on archive expand
 * some little code clean up
 *
 * Revision 1.5  2000/09/21 15:25:24  rusty
 *
 * some verboseness added
 *
 * Revision 1.4  2000/09/21 11:27:44  rusty
 *
 * bug fixes
 *
 * Revision 1.3  2000/09/20 17:43:07  rusty
 *
 * why checksums of big files seem to be corrupted?
 *
 * Revision 1.2  2000/09/20 14:50:29  rusty
 *
 * directory reorganizing
 *
 * Revision 1.1  2000/09/20 14:10:22  rusty
 *
 * First working (basic) release
 *
 */

#include "far.h"
#include "unarchive.h"
#include "utils.h"

char *unarch_version = "$Revision: 1.13 $";

int far_extract(unsigned char *buffer, unsigned int bufsize)
{
	FAR_INFO		*info		= (FAR_INFO *)NULL;
	unsigned	int	headsize	= 0;
	int			result		= 0;

	if(far_file_cksum(buffer, bufsize) == -1)
	{
		return -1;
	}

	info = far_retr_head(buffer, bufsize, &headsize);

	result = far_extract_files(buffer, bufsize, info, headsize);

	return result;
}

FAR_INFO *far_retr_head(unsigned char *buffer, unsigned int bufsize,
			unsigned int *headsize)
{
	FAR_INFO	*result	= (FAR_INFO *)NULL;
	size_t		count	= 0;

	memcpy(headsize, buffer, sizeof(size_t));
	(*headsize) = ntohl(*headsize);
	result = calloc(1, (sizeof(FAR_INFO) * (*headsize)));

	count = sizeof(size_t);

	memcpy(result, (buffer + count), (*headsize));

	far_head_byteorder_conv(result, ((*headsize) / sizeof(FAR_INFO)));

	return result;
}

int far_extract_files(unsigned char *buffer, unsigned int bufsize,
			FAR_INFO *head, unsigned int headsize)
{
	int	result		= 0,
		n_entries	= 0,
		count		= 0;

	n_entries = (headsize / sizeof(FAR_INFO));

	buffer += (sizeof(size_t) + headsize);
	for(; count < n_entries ; count++, head++)
	{
		far_printf(stderr, "%s", head->path);

		if(S_ISDIR(head->entry_info.fst_mode))
		{
			result += far_makedir(head);
		}
		else if(S_ISLNK(head->entry_info.fst_mode))
		{
			result += far_symlink(head);

			far_printf(stderr, " -> %s", head->link_path);
		}
		else if(S_ISREG(head->entry_info.fst_mode))
		{
			result += far_create_file(head, buffer);
		}
		else if(S_ISCHR(head->entry_info.fst_mode)	||
				S_ISBLK(head->entry_info.fst_mode)	||
				S_ISFIFO(head->entry_info.fst_mode))
		{
			result += far_mknod(head);
		}

		far_printf(stderr, "\n");
	}

	return result;
}

int far_makedir(FAR_INFO *info)
{
	int		result	= 0;
	struct	utimbuf	utb;

	result += mkdir(info->path, info->entry_info.fst_mode);

	result += lchown(info->path, info->entry_info.fst_uid,
					info->entry_info.fst_gid);

	utb.actime = info->entry_info.fst_atime;
	utb.modtime = info->entry_info.fst_mtime;
	result += utime(info->path, &utb);

	return result;
}

int far_symlink(FAR_INFO *info)
{
	int		result	= 0;
	struct	utimbuf	utb;

	result += symlink(info->link_path, info->path);

	result += lchown(info->path, info->entry_info.fst_uid,
					info->entry_info.fst_gid);

	utb.actime = info->entry_info.fst_atime;
	utb.modtime = info->entry_info.fst_mtime;
	result += utime(info->path, &utb);

	return result;
}

int far_create_file(FAR_INFO *info, unsigned char *buffer)
{
	unsigned	int	result	= 0;
	unsigned	char	*fbuf	=  buffer;
	FILE			*fp	= (FILE *)NULL;
	struct	utimbuf		utb;

	fbuf += info->offset;

	if(far_checksum(info, fbuf))
	{
		return -1;
	}

	fp = fopen(info->path, "w");

	if(fp)
	{
		fwrite(fbuf, info->entry_info.fst_size, 1, fp);
		fclose(fp);

		result += chmod(info->path, info->entry_info.fst_mode);
		result += lchown(info->path, info->entry_info.fst_uid,
						info->entry_info.fst_gid);

		utb.actime = info->entry_info.fst_atime;
		utb.modtime = info->entry_info.fst_mtime;
		result += utime(info->path, &utb);
	}
	else
	{
		result = -1;
	}

	return result;
}

int far_mknod(FAR_INFO *info)
{
	int		result	= 0;
	struct	utimbuf	utb;

	result += mknod(info->path, info->entry_info.fst_mode,
					info->entry_info.fst_rdev);

	result += lchown(info->path, info->entry_info.fst_uid,
					info->entry_info.fst_gid);

	utb.actime = info->entry_info.fst_atime;
	utb.modtime = info->entry_info.fst_mtime;
	result += utime(info->path, &utb);

	return result;
}

void far_head_byteorder_conv(FAR_INFO *info, int n)
{
	int	count	= 0;

	for(count = 0 ; ((count < n) && info) ; count++, info++)
	{
		far_byteorder_backconv(info);
	}
}

